;@***h* SOLAR_IRRADIANCE_FCDR/get_mg_index.pro
;
; NAME
;   get_mg_index
;
; PURPOSE
;   The get_mg_index.pro is a function that parses a time-series of the facular brightening index for the desired starting and ending date. 
;   
; DESCRIPTION
;   The get_mg_index.pro is a function that parses a time-series of the facular brightening index (actually, a proxy of the facular brightening index - Mg II) 
;   for the desired starting and ending date, and passes the results to the routine, process_irradiance.pro. 
;   Final data is parsed from LASP's time-series server, LaTiS, via an IDL net URL.
;      
; INPUTS
;   ymd1       - starting time range respective to midnight GMT of the given day, in 'yyyy-mm-dd' format.
;   ymd2       - ending time range respective to midnight GMT of the given day (i.e. in NOT inclusive), in 'yyyy-mm-dd' format.
;   final      - delegate to the LaTiS server for final released data.
;   cycle      - Directs data processing to facular brightening model inputs specific to solar cycle variability alone
;                (i.e. without the additional speculative secular change in facular brightening due to ephemeral regions on the Sun).
;   dev        - Data processing is delegated to processing routines for computing preliminary model input data. 
;   dataset    - name of LaTiS dataset to use in dev mode
;   version    - version and revision number of the NRLTSI2 and NRLSSI2 models (e.g., v02r00) 
;   _extra     - IDL keyword inheritance mechanism that can be used to pass keyword parameters by value
;                  
; OUTPUTS
;   data       - an IDL array containing the modified Julian date index and the Mg II index
;     
; AUTHOR
;   Odele Coddington, Laboratory for Atmospheric and Space Physics, Boulder, CO
;   Doug Lindholm, Laboratory for Atmospheric and Space Physics, Boulder, CO
;   Chris Lindholm, Laboratory for Atmospheric and Space Physics, Boulder, CO
;   Judith Lean, Space Science Division, Naval Research Laboratory, Washington, DC
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
;   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
;   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
;   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
;   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
;   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
;   SUPPORT TO USERS.
;
; REVISION HISTORY
;   06/04/2015 Initial Version prepared for NCDC
;   12/12/2016 Added capability to process facular brightening input based on processing "final", "prelim", or "dev" 
;                environment (/final, or /dev processing) and based on processing version (ex. "v02r00")
;   12/12/2016 Updated Robodoc header for v02r01 source code release.             
;
; USAGE
;   result=get_mg_index(ymd1,ymd2,final=final,cycle=cycle,dev=dev, dataset=dataset, version=version)
;
;@*****

;TODO: support missing ymd2, return just that day or no upper bound (latest)?
function get_mg_index_from_latis, ymd1, ymd2, final=final, cycle=cycle, $
                                  dev=dev, dataset=dataset, version=version
  ;add day to end time to make it inclusive
  end_date = mjd2iso_date(iso_date2mjdn(ymd2) + 1)

  ; Set default version.
  if ~keyword_set(version) then version = 'v02r01'

  ; Get the dataset name. It is important that the cases for more
  ; restrictive modes (like final) are handled first so they override
  ; less restrictive modes.
  if keyword_set(final) && keyword_set(cycle) then begin
    case version of
      'v02r00': dset = 'nrl2_facular_brightening_cycle_v02r00'
      'v02r01': dset = 'nrl2_facular_brightening_cycle'
    endcase
  endif else if keyword_set(final) then begin
    case version of
      'v02r00': dset = 'nrl2_facular_brightening_v02r00'
      'v02r01': dset = 'nrl2_facular_brightening_v02r01'
    endcase
  endif else if keyword_set(cycle) then begin
    dset = 'nrl2_facular_brightening_cycle'
  endif else if keyword_set(dev) then begin
    ; If the 'dev' keyword is set, use the 'dataset' keyword or
    ; default to the following.
    dset = n_elements(dataset) eq 0 ? 'bremen_composite_mgii' : dataset
  endif else dset = 'nrl2_facular_brightening'

  ;add query parameters
  query = 'convert(time,days since 1858-11-17)' ;convert times to MJD
  query += '&rename(time,MJD)' ;rename parameters to match the structures we expect here.

  ;get the data as an array of structures
  data = read_latis_data(dset, ymd1, end_date, query=query)
  return, data
end

function get_mg_index, ymd1, ymd2, _extra=_extra
  return, get_mg_index_from_latis(ymd1, ymd2, _extra=_extra)
end
