;@***h* SOLAR_IRRADIANCE_FCDR/nrl2_to_irradiance.pro
;
; NAME
;   nrl2_to_irradiance
;
; PURPOSE
;   The nrl2_to_irradiance.pro is the main driver procedure.  It calls subfunctions to compute Total Solar Irradiance (TSI) and 
;   Solar Spectral Irradiance (SSI) to write the data output to NetCDF4 formatted files. It calls a subfunction to extract the
;   speculative background irradiance component and add it to the estimates of irradiance uncertainty (derived from cycle-only
;   uncertainties alone). 
;
; DESCRIPTION
;   The nrl2_to_irradiance.pro is the main driver procedure.  It calls subfunctions to compute Total Solar Irradiance (TSI) and 
;   Solar Spectral Irradiance (SSI) to write the data output to NetCDF4 formatted files.It calls a subfunction to extract the
;   speculative background irradiance component and add it to the estimates of irradiance uncertainty (derived from cycle-only
;   uncertainties alone).
;      
; INPUTS
;   ymd1       - starting time range respective to midnight GMT of the given day, of the form 'yyyy-mm-dd'
;   ymd2       - ending time range respective to midnight GMT of the given day (i.e. in NOT inclusive), of the form 'yyyy-mm-dd'.
;   final      - Data processing is delegated to the LaTiS server for accessing final released values of model inputs. If not set, 
;                data processing is delegated to the LaTiS server for accessing preliminary resleased values of model inputs. 
;   dev        - Data processing is delegated to the LaTiS server for accessing development-mode  values of model inputs.  For use by Solar Irradiance CDR team only.
;   time_bin   - A value of 'year', 'month', or 'day' that defines the time-averaging performed for the given data records.
;               'day' is the default.
;   version    - version and revision number of the NRLTSI2 and NRLSSI2 models (e.g., v02r00)            
;   output_dir - path to data output directory.
;   facular_brightening_dataset - override LaTiS dataset used in dev mode for calculating facular brightening
;   
; OUTPUTS
; 
; AUTHOR
;   Odele Coddington, Laboratory for Atmospheric and Space Physics, Boulder, CO
;   Doug Lindholm, Laboratory for Atmospheric and Space Physics, Boulder, CO
;   Chris Lindholm, Laboratory for Atmospheric and Space Physics, Boulder, CO
;   Judith Lean, Space Science Division, Naval Research Laboratory, Washington, DC
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
;   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
;   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
;   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
;   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
;   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
;   SUPPORT TO USERS.
;
; REVISION HISTORY
;   06/04/2015 Initial Version prepared for NCDC
;   07/30/2015 Added dev keyword to function call (needed for process_irradiance.pro for "prelim" model inputs)
;   07/29/2016 Added calls to routines that compute the estimate of irradiance from model inputs with cycle-only components and to
;              separate out the background component of irradiance and add its magnitude to the irradiance uncertainty estimates.
;   12/12/2016 Updated Robodoc header for v02r01 source code release.           
;
; USAGE
;   nrl2_to_irradiance, ymd1, ymd2, final=final, dev=dev, time_bin=time_bin, version=version, output_dir=output_dir
;
;@*****

pro nrl2_to_irradiance, ymd1, ymd2, final=final, dev=dev, time_bin=time_bin, $
                        version=version, output_dir=output_dir, _extra=_extra

  ;Define the version of the data.
  ;If the version argument is not set, use the default.
  if n_elements(version) eq 0 then version = 'v02r01'  ;default to current final release version

  ; Default time bin is day.
  if n_elements(time_bin) eq 0 then time_bin = 'day'

  ; Run normal processing for the entire period.
  irradiance_data = $
     process_irradiance(ymd1, ymd2, version, final=final, $
                        dev=dev, time_bin=time_bin, _extra=_extra)

  ; Add cycle compontent to uncertainty if required.
  ;
  ; For v02r00 this will only add to the uncertainty for
  ; yearly-averaged data.
  ;
  ; For v02r01 this will add to the uncertainty for all time bins.
  irradiance_data_final = $
     process_cycle_if_required(irradiance_data, ymd1, ymd2, version, $
                               final=final, dev=dev, time_bin=time_bin, $
                               _extra=_extra)

  ;Write the data files.
  status = write_irradiance_data(ymd1, ymd2, irradiance_data_final, $
                                 version, final=final, time_bin=time_bin, $
                                 output_dir=output_dir)
end
