;@***h* SOLAR_IRRADIANCE_FCDR/process_cycle_if_required.pro
;
; NAME
;   process_cycle_if_required 
;
; PURPOSE
;   The process_cycle_if_required.pro procedure calls a series of functions to compute the Total Solar Irradiance (TSI) and
;   Solar Spectral Irradiance (SSI) using the NRLTSI2 and NRLSSI2 models using sunspot blocking and facular brightening model inputs
;   derived from solar cycle activity alone.  In a second step, a speculative "background-component" of solar variability speculated to 
;   result from ephemeral regions on the Sun is derived from the difference of these "cycle-only" irradiances with those derived
;   from the standard processing irradiances (that include cycle activity and a long-term, secular facular brightening component).  
;   Beginning v02r01, the magnitude of the speculative background irradiance component is added to the TSI and SSI uncertainties. In v02r00,
;   the magnitude of the speculative background irradiance composite was added to the uncertainties for yearly-averaged TSI values only.
;   Required when processing irradiances for time periods pre-dating 1978-11-06 (i.e. the date at which "global" irradiance due to 
;   facular brightening was available).
;
; DESCRIPTION
;   The process_cycle_if_required.pro procedure calls a series of functions to compute the Total Solar Irradiance (TSI) and
;   Solar Spectral Irradiance (SSI) using the NRLTSI2 and NRLSSI2 models using sunspot blocking and facular brightening model inputs
;   derived from solar cycle activity alone.  In a second step, a speculative "background-component" of solar variability speculated to 
;   result from ephemeral regions on the Sun is derived from the difference of these "cycle-only" irradiances with those derived
;   from the standard processing irradiances (that include cycle activity and a long-term, secular facular brightening component).  
;   Beginning v02r01, the magnitude of the speculative background irradiance component is added to the TSI and SSI uncertainties. In v02r00,
;   the magnitude of the speculative background irradiance composite was added to the uncertainties for yearly-averaged TSI values only.
;   Required when processing irradiances for time periods pre-dating 1978-11-06 (i.e. the date at which "global" irradiance due to 
;   facular brightening was available).
;   
; INPUTS
;   irr        - The standard processing irradiances obtained from process_irradiance.pro
;   ymd1       - starting time range respective to midnight GMT of the given day, of the form 'yyyy-mm-dd'
;   ymd2       - ending time range respective to midnight GMT of the given day (i.e. in NOT inclusive), of the form 'yyyy-mm-dd'.
;   version    - version and revision number of the NRLTSI2 and NRLSSI2 models (e.g., v02r00)
;   time_bin   - A value of 'year', 'month', or 'day' that defines the time-averaging performed for the given data records.
;               'day' is the default.
;   _extra     - IDL keyword inheritance mechanism that can be used to pass keyword parameters by value
;
; OUTPUTS
;   irradiance_data_final   - A structure containing the final irradiance data and the spectral bins:
;     mjd        - Modified Julian Date
;     iso        - iso 8601 formatted time
;     tsi        - Modeled Total Solar Irradiance
;     tsiunc     - Time-dependent Uncertainty in Modeled Total Solar Irradiance 
;                  (incl. magnitude of background irradiance component for processing dates prior to 1978-11-06)
;     ssi        - Modeled Solar Spectral Irradiance (in wavelength bins)
;     ssiunc     - Time- and wavelength-dependent Uncertainty in Modeled Solar Spectral Irradiance 
;                  (incl. magnitude of background irradiance component for processing dates prior to 1978-11-06)
;     ssitot     - Integral of the Modeled Solar Spectral Irradiance
;     nband      - number of spectral bands, for a variable wavelength grid, that the NRLSSI2 model bins 1 nm solar spectral irradiance onto.
;     bandcenter - the bandcenters (nm) of the variable wavelength grid.
;     bandwidth  - the bandwidths (delta wavelength, nm)  of the variable wavelength grid, centered on bandcenter.
;
; AUTHOR
;   Odele Coddington, Laboratory for Atmospheric and Space Physics, Boulder, CO
;   Chris Lindholm, Laboratory for Atmospheric and Space Physics, Boulder, CO
;   Judith Lean, Space Science Division, Naval Research Laboratory, Washington, DC
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
;   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
;   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
;   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
;   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
;   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
;   SUPPORT TO USERS.
;
; REVISION HISTORY
;   12/12/2016 Initial Version prepared for NCDC
;
; USAGE
;   result=process_cycle_if_required(irr, ymd1, ymd2, version, time_bin=time_bin, _extra=_extra)
;
;@*****


function process_cycle, irr, ymd1, ymd2, version, $
                        time_bin=time_bin, _extra=_extra
  compile_opt idl2, hidden

  ; We want to add the background component to the uncertainty using
  ; the cycle keyword through cycle_end. Because we perform binning
  ; prior to doing any calculations, we need to request enough data to
  ; fill the bin we want. Samples with and without the cycle component
  ; are identical after 1978-11-06.
  case time_bin of
    'day'  : cycle_end = '1978-11-06'
    'month': cycle_end = '1978-11-30'
    'year' : cycle_end = '1978-12-31'
  endcase
  cycle_end_mjd = iso_date2mjdn(cycle_end)
  mjd1 = iso_date2mjdn(ymd1)

  ; If the desired period starts before the cycle period ends, some of
  ; the processing must be done with the cycle keyword.
  if mjd1 le cycle_end_mjd then begin
    mjd3 = min([iso_date2mjdn(ymd2), cycle_end_mjd])
    ymd3 = mjd2iso_date(mjd3)
    cycle_data = $
       process_irradiance(ymd1, ymd3, version, final=final, dev=dev, $
                          time_bin=time_bin, _extra=_extra, /cycle)

    ind = where(irr.data.mjd le mjd3, comp=cind, /null)
    data_with_unc = $
       add_background_component_to_irradiance_uncertainty($
       irr.data[ind], cycle_data.data)
    irradiance_data_final = $
       {wavelength: irr.wavelength, $
        data: [data_with_unc, irr.data[cind]]}
  endif else irradiance_data_final = irr

  return, irradiance_data_final
end

function process_cycle_if_required, irr, ymd1, ymd2, version, $
                                    time_bin=time_bin, _extra=_extra
  compile_opt idl2

  case version of
    ; For v02r01 we want to add the speculative background component to the uncertainty
    ; for all time bins.
    'v02r01': irradiance_data_final = $
       process_cycle(irr, ymd1, ymd2, version, final=final, $
                     dev=dev, time_bin=time_bin, _extra=_extra)
    ; For v02r00 we want to add the speculative background component to the uncertainty
    ; for the yearly-averaged data only.
    'v02r00': if strcmp(time_bin, 'year', /fold_case) then begin
      irradiance_data_final = $
         process_cycle(irr, ymd1, ymd2, version, final=final, $
                       dev=dev, time_bin=time_bin, _extra=_extra)
    endif else irradiance_data_final = irr
    else: irradiance_data_final = irr
  endcase

  return, irradiance_data_final
end
