;@***h* SOLAR_IRRADIANCE_FCDR/write_irradiance_data.pro
;
; NAME
;   write_irradiance_data
;
; PURPOSE
;   The write_irradiance_data.pro function is the driver routine for writing irradiance output to netCDF4 output. It 
;   calls subroutines to a) dynamically create the filename with the NOAA NCDC filenaming conventions (create_filename.pro), 
;   b) write the TSI data record (write_tsi.pro), SSI data record (write_ssi.pro), and SSI uncertainty (write_ssi_unc.pro) data 
;   record (separate file from SSI data record), c) dynamically create the filename for the manifest file that accompanies each 
;   file delivered to NOAA NCDC (create_manifest.pro), and d) write the manifest data to the manifest file (write_to_manifest.pro).
;
; DESCRIPTION
;   The write_irradiance_data.pro function is the driver routine for writing irradiance output to netCDF4 output. It 
;   calls subroutines to a) dynamically create the filename with the NOAA NCDC filenaming conventions (create_filename.pro), 
;   b) write the TSI data record (write_tsi.pro), SSI data record (write_ssi.pro), and SSI uncertainty (write_ssi_unc.pro) data 
;   record (separate file from SSI data record), c) dynamically create the filename for the manifest file that accompanies each 
;   file delivered to NOAA NCDC (create_manifest.pro), and d) write the manifest data to the manifest file (write_to_manifest.pro).
;
; INPUTS
;   ymd1            - starting time range respective to midnight GMT of the given day, of the form 'yyyy-mm-dd'
;   ymd2            - ending time range respective to midnight GMT of the given day (i.e. in NOT inclusive), of the form 'yyyy-mm-dd'.
;   irradiance_data - a structure containing the following variables
;     data - a structure containing the following variables
;       mjd    - Modified Julian Date
;       iso    - iso 8601 formatted time
;       tsi    - Modeled Total Solar Irradiance
;       tsiunc - Uncertainty in total solar irradiance
;       ssi    - Modeled Solar Spectral Irradiance (in wavelength bins)
;       ssiunc - Uncertainty in solar spectral irradiance
;       ssitot - Integral of the Modeled Solar Spectral Irradiance
;     wavelength - a structure containing the following variables
;       nband      - number of spectral bands, for a variable wavelength
;                    grid, that the NRL2 model bins 1 nm solar spectral
;                    irradiance onto.
;       bandcenter - the bandcenters (nm) of the variable wavelength grid.
;       bandwidth  - the bandwidths (delta wavelength, nm) of the variable
;                    wavelength grid.
;   version         - version and revision number of the NRLTSI2 and NRLSSI2 models (e.g., v02r00)
;   final           - Produce final data if set
;   time_bin        - A value of 'year', 'month', or 'day' that defines the time-averaging performed for the given data records.
;                     'day' is the default.
;   output_dir - Directory path for irradiance files
;
; OUTPUTS
;
; AUTHOR
;   Odele Coddington, Laboratory for Atmospheric and Space Physics, Boulder, CO
;   Doug Lindholm, Laboratory for Atmospheric and Space Physics, Boulder, CO
;   Chris Lindholm, Laboratory for Atmospheric and Space Physics, Boulder, CO
;   Judith Lean, Space Science Division, Naval Research Laboratory, Washington, DC
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
;   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
;   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
;   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
;   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
;   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
;   SUPPORT TO USERS.
;
; REVISION HISTORY
;   06/04/2015 Initial Version prepared for NCDC
;   12/12/2016 Robodoc header updated for v02r01 source code release
;   04/05/2017 Writes ssi uncertainties to separate file only for v02r00 data.
;
; USAGE
;   result=write_irradiance_data(ymd1,ymd2,irradiance_data, version, final=final, time_bin=time_bin, output_dir=output_dir)
;
;@*****
function write_irradiance_data, ymd1, ymd2,irradiance_data, version, $
                                final=final, time_bin=time_bin, $
                                output_dir=output_dir

  creation_date = jd2iso_date(systime(/julian, /utc)) ;now as yyyy-mm-dd UTC

  ;Make sure we have a time bin defined. 'year', 'month', or 'day'
  if n_elements(time_bin) eq 0 then time_bin = 'day' ;default to daily

  ;Construct file names.
  tsi_file = create_filename(ymd1, ymd2, version, time_bin, $
                             final=final, /tsi)
  ssi_file = create_filename(ymd1, ymd2, version, time_bin, $
                             final=final, /ssi)
  ssiunc_file = create_filename(ymd1, ymd2, version, time_bin, $
                                final=final, /unc)

  ; Write TSI.
  write_tsi, ymd1, ymd2, creation_date, version, irradiance_data, $
             time_bin, tsi_file, output_dir, final=final

  ; Write SSI.
  write_ssi, ymd1, ymd2, creation_date, version, irradiance_data, $
             time_bin, ssi_file, output_dir, final=final

  if version eq 'v02r00' then begin
    ; Write SSI uncertainty to a separate file.
    write_ssi_unc, ymd1, ymd2, creation_date, version, irradiance_data, $
                   time_bin, ssiunc_file, output_dir, final=final
  endif
                 

  ;Dynamically determine file size (in bytes) and MD5 checksum and output to manifest file
  tsi_manifest = create_manifest(output_dir + tsi_file)
  ssi_manifest = create_manifest(output_dir + ssi_file)
  if version eq 'v02r00' then ssiunc_manifest = create_manifest(output_dir + ssiunc_file)

  ;Write the manifest data to file
  result = write_to_manifest(output_dir=output_dir, tsi_file, $
                             tsi_manifest.bytes, tsi_manifest.checksum, $
                             tsi_file + '.mnf')
  result = write_to_manifest(output_dir=output_dir, ssi_file, $
                             ssi_manifest.bytes, ssi_manifest.checksum, $
                             ssi_file + '.mnf')
  ; writes manifest file only when SSI uncertainties written as separate file (i.e for v2.0)                           
  if version eq 'v02r00' then result = write_to_manifest(output_dir=output_dir, ssiunc_file, $
                             ssiunc_manifest.bytes, ssiunc_manifest.checksum, $
                             ssiunc_file + '.mnf')

  ;TODO: error handling, return combined status
  return, 0
end
