;@***h* SOLAR_IRRADIANCE_FCDR/write_tsi.pro
;
; NAME
;   write_tsi.pro
;
; PURPOSE
;   Writes TSI data to a NetCDF file, filling in metadata appropriate
;   to the time binning.  Calls a sub procedure (write_tsi_global_attributes.pro) to dynamically populate
;   the global attributes of the title (based on version), data source (prelim or final data),
;   creation date, and history (i.e. when final data supercedes previously delivered prelim data).
;   
; DESCRIPTION
;   Writes TSI data to a NetCDF file, filling in metadata appropriate
;   to the time binning.  Calls a sub procedure (write_tsi_global_attributes.pro) to dynamically populate
;   the global attributes of the title (based on version), data source (prelim or final data),
;   creation date, and history (i.e. when final data supercedes previously delivered prelim data).
;   
; INPUTS
;   ymd1  - starting time  (yyyy-mm-dd)
;   ymd2  - ending time  (yyyy-mm-dd)
;   creation_date  - creation date (yyyy-mm-dd)
;   version - version and revision number of the NRLSSI2 model (e.g., v02r00)
;   irradiance_data - a structure containing the following variables
;     data - a structure containing the following variables
;       mjd    - Modified Julian Date
;       iso    - iso 8601 formatted time
;       tsi    - Modeled Total Solar Irradiance
;       tsiunc - Uncertainty in total solar irradiance
;       ssi    - Modeled Solar Spectral Irradiance (in wavelength bins)
;       ssiunc - Uncertainty in solar spectral irradiance
;       ssitot - Integral of the Modeled Solar Spectral Irradiance
;     wavelength - a structure containing the following variables
;       nband      - number of spectral bands, for a variable wavelength
;                    grid, that the NRL2 model bins 1 nm solar spectral
;                    irradiance onto.
;       bandcenter - the bandcenters (nm) of the variable wavelength grid.
;       bandwidth  - the bandwidths (delta wavelength, nm) of the variable
;                    wavelength grid.
;   time_bin   - one of 'day', 'month', or 'year'
;   filename   - filename
;   output_dir - Directory path for output file
;   final      - Produce final data if set
;
; OUTPUTS
; 
; AUTHOR
;   Odele Coddington, Laboratory for Atmospheric and Space Physics, Boulder, CO
;   Chris Lindholm, Laboratory for Atmospheric and Space Physics, Boulder, CO
;   Judith Lean, Space Science Division, Naval Research Laboratory, Washington, DC
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
;   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
;   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
;   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
;   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
;   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
;   SUPPORT TO USERS.
;
; REVISION HISTORY
;   06/04/2015 Initial Version prepared for NCDC
;   12/12/2016 Revised to allow for dynamically created global attributes for net CDF4 metadata
;   12/12/2016 Robodoc header updated for v02r01 source code release
;   03/23/2017 Updated metadata attributes for v02r01 source code release
;   04/05/2017 OC Revised date_modified and history attributes
;
; USAGE
;   result=write_tsi(ymd1, ymd2, creation_date, version, irradiance_data, time_bin, filename, output_dir, final=final)
;@*****

pro write_tsi_global_attributes, id, ymd1, ymd2, creation_date, version, $
                                 filename, time_bin, final=final
  compile_opt idl2, hidden

  NCDF_ATTPUT, id, /GLOBAL, "Conventions", "CF-1.6, ACDD-1.3", /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "title", $
               time_adjective(time_bin) + $
               " TSI calculated using NRL2 solar irradiance model", /CHAR

  if keyword_set(final) then begin
    src = 'NRLTSI2_' + version
  endif else begin
    src = 'NRLTSI2_' + version + '-preliminary'
  endelse
  NCDF_ATTPUT, id, /GLOBAL, "source", src, /CHAR

  NCDF_ATTPUT, id, /GLOBAL, "institution", "Naval Research Laboratory Space Science Division and Laboratory for Atmospheric and Space Physics", /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "standard_name_vocabulary", "CF Standard Name Table v29", /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "id", filename, /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "naming_authority", "gov.noaa.ncdc", /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "date_created", creation_date, /CHAR


  NCDF_ATTPUT, id, /GLOBAL, "license", "See Use Agreement for this CDR.", /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "summary", "This dataset contains total irradiance as a function of time created with the Naval Research Laboratory model for spectral and total irradiance. Total solar irradiance is the total, spectrally integrated energy input to the top of the Earth’s atmosphere, at a standard distance of one Astronomical Unit from the Sun. Its units are W per m2. The dataset is created by Judith Lean (Space Science Division, Naval Research Laboratory), and Odele Coddington, Doug Lindholm, Chris Lindholm, Peter Pilewskie, and Martin Snow (Laboratory for Atmospheric and Space Science, University of Colorado).", /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "keywords", "Earth Science &gt; Atmosphere &gt; Atmospheric Radiation &gt; Incoming Solar Radiation, Earth Science &gt; Atmosphere &gt; Atmospheric Radiation &gt; Solar Irradiance, Earth Science &gt; Atmosphere &gt; Atmospheric Radiation &gt; Solar Radiation, Earth Science &gt; Sun-Earth Interactions &gt; Solar Activity &gt; Solar Irradiance, Earth Science &gt; Paleoclimate &gt; Paleoclimate Reconstructions &gt; Solar Forcing/Insolation Reconstruction, Earth Science &gt; Sun-Earth Interactions &gt; Solar Activity &gt; Solar Irradiance, Earth Science &gt; Climate Indicators &gt; Sun-Earth Interactions &gt; Sunspot Activity &gt; Solar Flux, Earth Science &gt; Climate Indicators &gt; Paleoclimate Indicators &gt; Paleoclimate Reconstructions &gt; Solar Forcing/Insolation Reconstruction, Earth Science &gt; Climate Indicators &gt; Sun-Earth Interactions &gt; Solar Activity &gt; Solar Irradiance", /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "keywords_vocabulary", "Global Change Master Directory (GCMD) Science and Services Keywords", /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "cdm_data_type", "Any", /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "time_coverage_start", ymd1, /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "time_coverage_end", ymd2, /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "cdr_program", "NOAA Climate Data Record Program", /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "cdr_variable", "TSI", /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "metadata_link", "http://doi.org/10.7289/V55B00C1", /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "product_version", version, /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "geospatial_lat_min","-90.0", /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "geospatial_lat_max"," 90.0", /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "geospatial_lon_min","-180.0", /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "geospatial_lon_max"," 180.0", /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "spatial_resolution", "N/A", /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "contributor_name", "Judith Lean, Odele Coddington, Peter Pilewskie", /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "contributor_role", "Principal Investigator and originator of total and spectral solar irradiance model, Principal Investigator and Point-of-Contact for FCDR output data, Co-Principal Investigator ensuring overall integrity of the data product", /CHAR
  NCDF_ATTPUT, id, /GLOBAL, "creator_name", "Odele Coddington, Judith Lean, Peter Pilewskie", /CHAR
  
  if version eq 'v02r01' then begin
    if keyword_set(final) then begin
      case time_bin of
        'year': history = "This file supersedes version v02r00 data." 
        ;'year': history = "This file supersedes version v02r00 data and the previous year's v02r01 final data file by adding an additional sample."
        else: history = "This file supersedes version v02r00 data." 
        ;else: history = "This file supersedes version v02r00 data and preliminary v02r01 data for the year."
      endcase
      NCDF_ATTPUT, id, /GLOBAL, "history", history, /CHAR
      ;NCDF_ATTPUT, id, /GLOBAL, "date_modified", creation_date, /CHAR ;uncomment when we transition to "typical" processing (i.e replacing prelim data with final data)
    endif   
  endif
  
  if version eq 'v02r00' then begin
    if keyword_set(final) then begin
      case time_bin of
        'year': history = "This file supersedes the previous year's final data file by adding an additional sample."
        else: history = "This file supersedes preliminary data for the year."
      endcase
      NCDF_ATTPUT, id, /GLOBAL, "history", history, /CHAR
      NCDF_ATTPUT, id, /GLOBAL, "date_modified", creation_date, /CHAR
    endif
  endif 
 
end


pro write_tsi, ymd1, ymd2, creation_date, version, irradiance_data, $
               time_bin, filename, output_dir, final=final
  compile_opt idl2

  ; Extract data component.
  data = irradiance_data.data

  ; Define missing value and replace NaNs in the modeled data.
  missing_value = -99.0
  tsi = replace_nan_with_value(data.tsi, missing_value)
  tsiunc = replace_nan_with_value(data.tsiunc, missing_value)

  ; Define the bounds for each time bin.
  day_zero_mjd = iso_date2mjdn('1610-01-01')
  time_bounds = get_time_bounds(time_bin, data.mjd)

  ; Create NetCDF file.
  file_path = output_dir + path_sep() + filename
  id = ncdf_create(file_path, /noclobber, /netcdf4_format)

  write_tsi_global_attributes, id, ymd1, ymd2, creation_date, version, $
                               filename, time_bin, final=final

  ; Define Dimensions
  tid = NCDF_DIMDEF(id, 'time', /UNLIMITED) ;time series
  bid = NCDF_DIMDEF(id, 'bounds', 2)        ;time bounds dimension

  ; Variable Attributes
  x1id = NCDF_VARDEF(id, 'TSI', [tid], /FLOAT, CHUNK=1)
  NCDF_ATTPUT, id, x1id, 'long_name', $
               'NOAA Climate Data Record of ' + time_adjective(time_bin) + $
               ' Total Solar Irradiance (W m-2)', /CHAR
  NCDF_ATTPUT, id, x1id, 'standard_name', 'solar_irradiance', /CHAR
  NCDF_ATTPUT, id, x1id, 'units', 'W m-2', /CHAR
  NCDF_ATTPUT, id, x1id, 'cell_methods', 'time: mean', /CHAR
  NCDF_ATTPUT, id, x1id, 'ancillary_variables', 'TSI_UNC', /CHAR
  NCDF_ATTPUT, id, x1id, 'missing_value', missing_value

  x3id = NCDF_VARDEF(id, 'time', [tid], /FLOAT, CHUNK=1)
  NCDF_ATTPUT, id, x3id, 'units', 'days since 1610-01-01 00:00:00', /CHAR
  NCDF_ATTPUT, id, x3id, 'standard_name', 'time', /CHAR
  NCDF_ATTPUT, id, x3id, 'axis', 'T', /CHAR
  NCDF_ATTPUT, id, x3id, 'bounds', 'time_bnds', /CHAR

  x4id = NCDF_VARDEF(id, 'TSI_UNC', [tid], /FLOAT, CHUNK=1)
  NCDF_ATTPUT, id, x4id, 'long_name', $
               'Uncertainty in ' + time_adjective(time_bin) + $
               ' Total Solar Irradiance (W m-2)', /CHAR
  NCDF_ATTPUT, id, x4id, 'units', 'W m-2', /CHAR
  NCDF_ATTPUT, id, x4id, 'missing_value', missing_value

  x5id = NCDF_VARDEF(id, 'time_bnds', [bid,tid], /FLOAT)
  NCDF_ATTPUT, id, x5id, 'long_name', $
               'Minimum (inclusive) and maximum (exclusive) dates included in the time averaging', /CHAR
  NCDF_ATTPUT, id, x5id, 'units', 'days since 1610-01-01 00:00:00', /CHAR

  ; Put file in data mode
  NCDF_CONTROL, id, /ENDEF

  ; Insert data
  NCDF_VARPUT, id, x3id, data.mjd - day_zero_mjd
  NCDF_VARPUT, id, x1id, tsi
  NCDF_VARPUT, id, x4id, tsiunc
  NCDF_VARPUT, id, x5id, time_bounds - day_zero_mjd

  ; Close the NetCDF file
  NCDF_CLOSE, id
end
