;@***h* TSI_FCDR/get_sunspot_data.pro
; 
; NAME
;   get_sunspot_data.pro
;
; PURPOSE
;   Aquire U.S. Air Force white Light sunspot region data from a NOAA/NGDC web repository- point of contact: Bill Denig.
;   http://www.ngdc.noaa.gov/stp/space-weather/solar-data/solar-features/sunspot-regions/usaf_mwl/
;
; DESCRIPTION
;   This routine is called from the function, process_sunspot_blocking.pro for the case of preliminary data.
;   It accesses the above url and parses the given record into a data structure. Each record is for a sunspot group 
;   measurement. The time (YYMMDD), solar latitude and longitude, sunspot area, and station ID are parsed from each record.
;   When the "final" keyword is set, data processing relegated to the LaTiS server for accessing a record of the final released 
;   values of model inputs. 
;   
; INPUTS
;   ymd1                - starting time range respective to midnight GMT of the given day, in Modified Julian day (converted from 'yyyy-mm-dd' in main driver).
;   ymd2                - ending time range respective to midnight GMT of the given day  
;   stations = stations - Optional keyword to restrict sunspot darkening index to specified monitoring stations in the USAF white light network.
;                         If omitted (default), these stations are used: 'LEAR','CULG','SVTO','RAMY','BOUL','HOLL','PALE','MANI','ATHN'. 
;                         Used for QA analysis.
;   final=final         - Final sunspot blocking model input data is parsed from LASP's time-series server, LaTiS, via an IDL net URL.
;   
; OUTPUTS
;   'data' - a structure containing (for each record in the USAF data) is returned to process_sunspot_blocking
;     mjd      - Modified Julian Date 
;     lat      - latitude of sunspot group
;     lon      - longitude of sunspot group
;     group    - sunspot group number 
;     area     - sunspot area
;     station  - station name 
;
; AUTHOR
;   Odele Coddington, Laboratory for Atmospheric and Space Physics, Boulder, CO
;   Doug Lindholm, Laboratory for Atmospheric and Space Physics, Boulder, CO
;   Chris Lindholm, Laboratory for Atmospheric and Space Physics, Boulder, CO
;   Judith Lean, Space Science Division, Naval Research Laboratory, Washington, DC
;   
; COPYRIGHT 
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
;   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
;   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
;   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
;   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
;   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
;   SUPPORT TO USERS.
;
; REVISION HISTORY
;   06/04/2015 Initial Version prepared for NCDC
;   12/12/2016 Updated Robodoc header for v02r01 source code release.
; 
; USAGE
;   result=get_sunspot_data(ymd1, ymd2, stations=stations)
;
;@*****

;-----------------------------------------------------------------------------
; Adjust records from LaTiS.
function adjust_record, x

  ; Replace missing area values.
  area = isa(x.area, /string) ? !VALUES.D_NAN : x.area

  ; Set the sign of the latitude based on the hemisphere.
  lat = x.lathem eq 'N' ? x.lat : -x.lat

  ; Set the sign of the longitude based on the hemisphere.
  lon = x.lonhem eq 'W' ? x.lon : -x.lon

  ; Create result structure.
  result = {mjd: x.mjd,     $
            lat: lat,       $
            lon: lon,       $
            group: x.group, $
            area: area,     $
            station: x.station}

  return, result
end

;-----------------------------------------------------------------------------
; Get data from LaTiS.
function get_sunspot_data_from_latis, ymd1, ymd2, final=final
  ;ymd2 is inclusive.
  ;Since a datetime without time is typically interpreted as midnight (time = 00:00)
  ;we will manage the addition of one day internal to these algorithms.
  ;ymd values for time range are expected to be dates of the form 'yyyy-mm-dd'.

  ;LaTiS assume that yyyy-MM-dd is midnight of that date so we need to add a
  ;day to make the request inclusive.
  end_date = mjd2iso_date(iso_date2mjdn(ymd2) + 1)

  ; Query LaTiS for data, converting time to MJD.
  if keyword_set(final) then begin
    dataset = 'usaf_sunspot_regions_final'
  endif else begin
    dataset = 'usaf_sunspot_regions'
  endelse
  query = 'convert(time,days since 1858-11-17)&rename(time,MJD)'
  data = read_latis_data(dataset, ymd1, end_date, query=query, /tolist)

  ; Reformat each record to match format expected by callers.
  res = list()
  foreach d, data do begin
    res.add, adjust_record(d)
  endforeach

  return, res.toArray()
end

;-----------------------------------------------------------------------------
; Use this routine to get USAF sunspot region data.
function get_sunspot_data, ymd1, ymd2, stations=stations, final=final
  ;TODO: add keyword to get data from alternate source?
  data = get_sunspot_data_from_latis(ymd1, ymd2, final=final)

  ;Keep only requested stations
  if n_elements(stations) gt 0 then begin
    list = List() ;keep a list of records with station in stations
    for i=0, n_elements(data)-1 do begin
      station = strupcase(data[i].station)
      w = where(stations eq station, n)
      if n eq 1 then list.add, data[i]
    endfor

    data = list.toArray()
  endif

  return, data
end
