;@***h* SOLAR_IRRADIANCE_FCDR/read_latis_data.pro
;
; NAME
;   read_latis_data
;
; PURPOSE
;   The read_latis_data.pro is a function that gets data from the LASP Time Series Server as a list of structures
;
; DESCRIPTION
;   The read_latis_data.pro is a function that gets data from the LASP Time Series Server as a list of structures
;   Data is parsed from LASP's time-series server, LaTiS, via an IDL net URL.
;
; INPUTS
;   dataset    - The name of the dataset
;   start_time - starting time range to aquire the data, in 'yyyy-mm-dd' format.
;   end_time   - ending time range to aquire the data, in 'yyyy-mm-dd' format.
;   host       - name of the server host
;   port       - the port on the server machine
;   base_path  - Directory path on server to the dataset
;   query      - query parameters used to convert time or rename parameters to match the structures in LaTiS.
;   tolist     - Return results as a list of structures.
;
; OUTPUTS
;   list       - an IDL list containing the dataset values
;
; AUTHOR
;   Odele Coddington, Laboratory for Atmospheric and Space Physics, Boulder, CO
;   Doug Lindholm, Laboratory for Atmospheric and Space Physics, Boulder, CO
;   Judith Lean, Space Science Division, Naval Research Laboratory, Washington, DC
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
;   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
;   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
;   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
;   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
;   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
;   SUPPORT TO USERS.
;
; REVISION HISTORY
;   06/04/2015 Initial Version prepared for NCDC
;
; USAGE
;   result=read_latis_data(dataset, start_time, end_time, host=host, port=port, base_path=base_path, query=query)
;
;@*****
function read_latis_data, dataset, start_time, end_time, host=host, port=port, $
                          base_path=base_path, query=query, tolist=tolist
  ;TODO: return catalog if no dataset if defined?
  ;TODO: error handling (see MMS SITL tools)
  ;TODO: support authentication (see MMS SITL tools)
  ;TODO: add read_latis_metadata? get_latis_catalog?
  ;TODO: add /TOLIST to get lists instead of arrays, likewise for hash, dict?
  ;TODO: auto update version for header
  version = '1.0.0'
  
  ;if not keyword_set(host)      then host = 'lasp.colorado.edu'
  if not keyword_set(host)      then host = 'lisird-dev'
  ;if not keyword_set(port)      then port = 80
  if not keyword_set(port)      then port = 8080
  if not keyword_set(base_path) then base_path = 'lisird3/latis'
  
  ;Build the LaTiS query string
  url_query = ''
  if n_elements(start_time) gt 0  then url_query = url_query + '&time>=' + start_time
  if n_elements(end_time)   gt 0  then url_query = url_query + '&time<'  + end_time
  if n_elements(query)      gt 0  then url_query = url_query + '&' + query

  ;Make User-Agent header
  v = !VERSION
  header = 'User-Agent: latis-idl-client/' + version $
    + ' (IDL/' + v.release + ' ' + v.os + ' ' + v.arch + ')'

  ;Construct and execute the request
  netUrl = OBJ_NEW('IDLnetUrl')
  netUrl->SetProperty, URL_HOST  = host
  netUrl->SetProperty, URL_PORT  = port
  netURL->SetProperty, URL_PATH  = base_path + '/' + dataset + '.json' ;TODO: use content negotiation
  netURL->SetProperty, URL_QUERY = url_query
  netURL->SetProperty, HEADERS   = header
  lines = netURL->Get(/string_array)
  OBJ_DESTROY, netUrl

  ;combine all the lines into a single string
  ;json = lines.Reduce(Lambda(x,y: x+y)) ;requires IDL 8.4
  json = ''
  for i = 0, n_elements(lines)-1 do json += lines[i]

  ; If /toList wasn't specified, tell json_parse to return an array.
  toarray = ~keyword_set(tolist)

  ;parse the json response
  data = json_parse(json, /TOSTRUCT, toarray=toarray)
  ;Outer struct with dataset name.
  ;Inner struct with "samples" (for the LaTiS Function)
  ;Then just an array of Structs with an element for each parameter. Return that.
  samples = data.(0).(0)
  return, samples

end
